@@ -46,6 +46,7 @@ class PGG_diffusion:
         ):
 
         import collections
+        import numpy as np 
         self._noise_var = noise_var
         self._G = G
         self._synergy = synergy
@@ -93,17 +94,18 @@ class PGG_diffusion:
         else:
             noise = 0.0
 
-        if self._G.node[dealer]['strategy'] == 'C':
+        if self._G.node[dealer]['strategy'] == 'C' and self._G.node[dealer]['ready_to_sent'] == 1:
+        #if self._G.node[dealer]['strategy'] == 'C':
             Ni = [self._G.node[nd]['strategy'] for nd in self._G.neighbors(dealer)].count('C')
             cost_y = 1.0/(Ni+1)
         else:
             cost_y = 0
 
         cost_i = 0
-        for i in self._G.neighbors(dealer):
+        for i in dealer_neighborhood:
             if self._G.node[i]['strategy'] == 'C' and self._G.node[i]['ready_to_sent'] == 1:
                 Nj = [self._G.node[nd]['strategy'] for nd in self._G.neighbors(i)].count('C')
-                cost_i += 1.0 / (Nj + 1)
+                cost_i += 1.0 / (Nj+1)
 
         Nc = [self._G.node[nd]['strategy'] for nd in self._G.neighbors(dealer)].count('C')
         self._G.node[dealer]['payoff'] = (float(self._synergy)/(Nc+1)) * cost_i - cost_y + noise
@@ -117,17 +119,20 @@ class PGG_diffusion:
             if H.node[node]['state'] == 'I' and H.node[node]['strategy'] == 'C':
                 neighbors = H.neighbors(node)
                 pkt = -1
+                RD.shuffle(neighbors)
                 for neighbor in neighbors:
                     if pkt == -1:
                         ready_to_sent = list(set(H.node[node]['time_stamp']) - set(H.node[neighbor]['time_stamp']))
                         if len(ready_to_sent) > 0:
                             pkt = choice(ready_to_sent)
-                    else:
-                        if pkt >= 0:
-                            if not pkt in H.node[neighbor]['time_stamp']:
-                                H.node[neighbor]['time_stamp'].append(pkt)
-                                H.node[neighbor]['state'] = 'I'
+                            H.node[neighbor]['time_stamp'].append(pkt)
+                            H.node[neighbor]['state'] = 'I'
                             H.node[node]['ready_to_sent'] = 1
+                            for nn in neighbors:
+                                if not (pkt in H.node[nn]['time_stamp']):
+                                    H.node[nn]['time_stamp'].append(pkt)
+                                    H.node[nn]['state'] = 'I'
+                            break
         self._G = H.copy()
 
     def run_game(self, nb_simulation_step=None):
@@ -136,11 +141,37 @@ class PGG_diffusion:
         else:
             nb_step = self._nb_simulation_step
         for i in xrange(int(nb_step)):
-            self.reset_send_update()
             self.diffusion_step()
+            for dealer in self._G.nodes_iter():
+                self.compute_payoffs(dealer)
             self.store_cooperator_defector_counter()
             self.updateStrategy()
             self.reset_payoffs()
+
+            #########
+            # Print Debug 
+            #
+            # NbInfected = 0
+            # NbPacket = 0
+            # nb_update = 0
+            # for nd in self._G.nodes():
+            #     NbPacket += len(self._G.node[nd]['time_stamp'])
+            #     if self._G.node[nd]['ready_to_sent'] == 1:
+            #        nb_update += 1
+            #     if self._G.node[nd]['state'] == 'I':
+            #         NbInfected += 1
+            # finalState = self._nb_node * self._buffer_size
+
+            # print 'NbPacket', 100*float(NbPacket)/finalState , 'Nb Update: ', nb_update, ' NbInfected ', NbInfected
+            #########
+            # End Print Debug 
+            #
+
+
+            self.reset_send_update()
+            for infected in self._infected_node:
+                self.set_infected(infected, range(self._buffer_size))
+        
         nb_coop = sum(self._nb_coop) / len(self._nb_coop)
         return float(nb_coop) / len(self._G.nodes())
 
@@ -206,6 +237,7 @@ class PGG_diffusion:
         self._G.node[node_id]['state'] = 'I'
         self._G.node[node_id]['strategy'] = 'C'
         self._G.node[node_id]['time_stamp'] = time_stamp
+        self._G.node[node_id]['ready_to_sent'] = 1
 
     def set_sent_update(self, sent_update):
        for (key, val) in sent_update.iteritems():
